package com.tzld.piaoquan.recommend.server.service;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.google.common.base.Stopwatch;
import com.google.common.base.Strings;
import com.tzld.piaoquan.recommend.server.common.base.RequestContext;
import com.tzld.piaoquan.recommend.server.common.enums.AppTypeEnum;
import com.tzld.piaoquan.recommend.server.gen.common.Result;
import com.tzld.piaoquan.recommend.server.gen.recommend.RecommendRequest;
import com.tzld.piaoquan.recommend.server.gen.recommend.RecommendResponse;
import com.tzld.piaoquan.recommend.server.gen.recommend.VideoProto;
import com.tzld.piaoquan.recommend.server.model.RecommendParam;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.flowpool.FlowPoolConfigService;
import com.tzld.piaoquan.recommend.server.service.flowpool.FlowPoolConstants;
import com.tzld.piaoquan.recommend.server.service.flowpool.FlowPoolService;
import com.tzld.piaoquan.recommend.server.service.rank.RankParam;
import com.tzld.piaoquan.recommend.server.service.rank.RankResult;
import com.tzld.piaoquan.recommend.server.service.rank.RankService;
import com.tzld.piaoquan.recommend.server.service.recall.RecallParam;
import com.tzld.piaoquan.recommend.server.service.recall.RecallResult;
import com.tzld.piaoquan.recommend.server.service.recall.RecallService;
import com.tzld.piaoquan.recommend.server.service.recall.strategy.*;
import com.tzld.piaoquan.recommend.server.util.CommonCollectionUtils;
import com.tzld.piaoquan.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.RandomUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import com.tzld.piaoquan.recommend.server.util.ParserUtils;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RecommendService {

    @Autowired
    private RedisTemplate<String, String> redisTemplate;


    private Map<Integer, String> ab_initial_config_map = new HashMap<>();

    @ApolloJsonValue("${ab_exp_code:{}}")
    private Map<String, Map<String, String>> abExpCodeMap;

    @ApolloJsonValue("${app_ab_code:{}}")
    private Map<String, Map<String, Map<String, String>>> appAbCodeMap;

    @ApolloJsonValue("${flow.pool.ids:[7, 8, 9, 10, 11, 12, 13, 14, 15, 16]}")
    private List<Integer> flowPoolIds;
    @ApolloJsonValue("${city_code:[]}")
    private Set<String> cityCodes;

    @Autowired
    private FlowPoolConfigService flowPoolConfigService;
    @Autowired
    private RecallService recallService;
    @Autowired
    private RankService rankService;
    @Autowired
    private PreViewedService preViewedService;
    @Autowired
    private FlowPoolService flowPoolService;

    @PostConstruct
    public void init() {
        ab_initial_config_map.put(AppTypeEnum.VLOG.getCode(), "095");
        ab_initial_config_map.put(AppTypeEnum.LOVE_LIVE.getCode(), "144");
        ab_initial_config_map.put(AppTypeEnum.LONG_VIDEO.getCode(), "121");
        ab_initial_config_map.put(AppTypeEnum.LAO_HAO_KAN_VIDEO.getCode(), "074");
        ab_initial_config_map.put(AppTypeEnum.ZUI_JING_QI.getCode(), "069");
    }

    public RecommendResponse homepageRecommend(RecommendRequest request) {
        return recommend(request, 0);
    }

    public RecommendResponse relevantRecommend(RecommendRequest request) {
        return recommend(request, 1);
    }

    public RecommendResponse recommend(RecommendRequest request, int recommendType) {
        if (request == null) {
            return RecommendResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1).setMessage("success"))
                    .build();
        }
        if (request.getVersionAuditStatus() == 1) {
            return specialMidRecommend(request);
        }
        Stopwatch stopwatch = Stopwatch.createStarted();
        if (StringUtils.isNotBlank(request.getMid())
                && redisTemplate.opsForSet().isMember("special:mid", request.getMid())) {
            return specialMidRecommend(request);
        }
        log.info("sprcial mid cost={}", stopwatch.stop().elapsed(TimeUnit.MILLISECONDS));
        stopwatch.reset().start();

        RecommendParam param = genRecommendParam(request, recommendType);
        log.info("genRecommendParam={},genRecommendParam cost={}", JSONUtils.toJson(param),
                stopwatch.stop().elapsed(TimeUnit.MILLISECONDS));
        stopwatch.reset().start();
        List<Video> videos = videoRecommend(param);
        log.info("videoRecommend={}, videoRecommend cost={}", JSONUtils.toJson(videos),
                stopwatch.stop().elapsed(TimeUnit.MILLISECONDS));
        stopwatch.reset().start();
        updateCache(request, param, videos);
        log.info("updateCache={}, updateCache cost={}", JSONUtils.toJson(videos), stopwatch.stop().elapsed(TimeUnit.MILLISECONDS));

        // 更新position
        List<VideoProto> vps = new ArrayList<>();
        for (int i = 0; i < videos.size(); i++) {
            vps.add(VideoProto.newBuilder()
                    .setPosition(i + 1)
                    .setPushFrom(Strings.nullToEmpty(videos.get(i).getPushFrom()))
                    .setAbCode(Strings.nullToEmpty(videos.get(i).getAbCode()))
                    .setVideoId(videos.get(i).getVideoId())
                    .setRovScore(videos.get(i).getRovScore())
                    .setSortScore(videos.get(i).getSortScore())
                    .setFlowPool(Strings.nullToEmpty(videos.get(i).getFlowPool()))
                    .setIsInFlowPool(videos.get(i).isInFlowPool() ? 1 : 0)
                    .setRand(videos.get(i).getRand())
                    .build());
        }

        return RecommendResponse.newBuilder()
                .setResult(Result.newBuilder().setCode(1).setMessage("success"))
                .addAllVideo(vps)
                .build();

    }

    private RecommendResponse specialMidRecommend(RecommendRequest request) {
        log.info("hit special mid recommend request={}", JSONUtils.toJson(request));
        if (request == null) {
            return RecommendResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1).setMessage("success"))
                    .build();
        }

        RecallParam recallParam = new RecallParam();
        recallParam.setAppType(request.getAppType());
        recallParam.setMid(request.getMid());
        recallParam.setSpecialRecommend(true);
        recallParam.setSize(request.getSize());

        RecallResult recallResult = recallService.recall(recallParam);
        log.info("recallParam {}, recallResult {}", JSONUtils.toJson(recallParam), JSONUtils.toJson(recallResult));

        RankParam rankParam = new RankParam();
        rankParam.setRecallResult(recallResult);
        rankParam.setSize(request.getSize());
        rankParam.setSpecialRecommend(true);

        RankResult rankResult = rankService.rank(rankParam);
        log.info("rankParam, rankResult {}", JSONUtils.toJson(rankParam), JSONUtils.toJson(rankResult));

        if (rankResult == null || CollectionUtils.isEmpty(rankResult.getVideos())) {
            return RecommendResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1).setMessage("success"))
                    .build();
        }

        // 只返回size条数据
        List<Video> videos = rankResult.getVideos();
        if (recallParam.getSize() < rankResult.getVideos().size()) {
            videos = rankResult.getVideos().subList(0, recallParam.getSize());
        }

        if (CollectionUtils.isNotEmpty(videos)) {
            Video lastVideo = videos.get(videos.size() - 1);
            log.info("update recall last video  lastVideo={}", JSONUtils.toJson(lastVideo));
            redisTemplate.opsForValue().set(lastVideo.getLastVideoKey(), String.valueOf(lastVideo.getVideoId()),
                    1, TimeUnit.DAYS);
        }

        return RecommendResponse.newBuilder()
                .setResult(Result.newBuilder().setCode(1).setMessage("success"))
                .addAllVideo(CommonCollectionUtils.toList(videos, v -> VideoProto.newBuilder()
                        .setPushFrom(Strings.nullToEmpty(v.getPushFrom()))
                        .setAbCode(Strings.nullToEmpty(v.getAbCode()))
                        .setVideoId(v.getVideoId())
                        .setRovScore(v.getRovScore())
                        .build()))
                .build();
    }

    private RecommendParam genRecommendParam(RecommendRequest request, int recommendType) {
        RecommendParam param = new RecommendParam();
        param.setTopK(3);
        param.setFlowPoolP(0.3);
        param.setMid(request.getMid());
        param.setUid(request.getUid());
        param.setSize(request.getSize());
        param.setVideoId(request.getVideoId());
        param.setCityCode(request.getCityCode());
        param.setProvinceCode(request.getProvinceCode());
        param.setAppType(request.getAppType());

        String abInitialConfig = ab_initial_config_map.getOrDefault(request.getAppType(), "095-1");
        Map<String, String> abExpCode = abExpCodeMap.getOrDefault(abInitialConfig, Collections.emptyMap());
        param.setAbCode(abExpCode.get("ab_code"));
        param.setRuleKey(abExpCode.get("rule_key"));
        param.setDataKey(abExpCode.get("data_key"));
        param.setRankKeyPrefix("rank:score1:");

        // 实验配置
        if (request.getAbExpCodeCount() > 0) {
            Set<String> abExpCodes = request.getAbExpCodeList().stream().collect(Collectors.toSet());
            // 流量池视频分发概率
            if (abExpCodes.contains("211")) {
                param.setFlowPoolP(0.9);
            } else if (abExpCodes.contains("221")) {
                param.setFlowPoolP(0.7);
            } else if (abExpCodes.contains("299")) {
                param.setFlowPoolP(0.5);
            } else if (abExpCodes.contains("300")) {
                param.setFlowPoolP(0.4);
            } else if (abExpCodes.contains("301")) {
                param.setFlowPoolP(0.6);
            } else if (abExpCodes.contains("339")) {
                param.setFlowPoolP(0);
            }

            for (Map.Entry<String, Map<String, String>> entry : abExpCodeMap.entrySet()) {
                if (abExpCodes.contains(entry.getKey())) {
                    param.setAbCode(entry.getValue().get("ab_code"));
                    param.setRuleKey(entry.getValue().get("rule_key"));
                    param.setDataKey(entry.getValue().get("data_key"));
                    param.setRankKeyPrefix(StringUtils.isNotBlank(entry.getValue().get("rank_key_prefix"))
                            ? entry.getValue().get("rank_key_prefix")
                            : "rank:score1:");
                    break;
                }
            }
        }

        // app实验组
        if (request.getEventIdCount() > 0) {
            if (recommendType == 0) {
                String eventId = request.getEventIdMap().get("10003");
                Map<String, Map<String, String>> appAbCode = appAbCodeMap.get("10003");
                if (StringUtils.isNotBlank(eventId)
                        && appAbCode != null
                        && appAbCode.containsKey(eventId)) {
                    param.setAbCode(appAbCode.get(eventId).get("ab_code"));
                    param.setRuleKey(appAbCode.get(eventId).get("rule_key"));
                    param.setDataKey(appAbCode.get(eventId).get("data_key"));
                }
            }
        }

        // 流量池分发实验组划分
        int flowPoolIdChoice = flowPoolIds.get(RandomUtils.nextInt(0, flowPoolIds.size() - 1));
        param.setFlowPoolId(flowPoolIdChoice);
        param.setFlowPoolAbtestGroup("control_group");
        Map<String, List<Integer>> flowPoolConfig = flowPoolConfigService.getFlowPoolConfig();
        for (Map.Entry<String, List<Integer>> entry : flowPoolConfig.entrySet()) {
            if (entry.getValue().contains(flowPoolIdChoice)) {
                param.setFlowPoolAbtestGroup(entry.getKey());
            }
        }

        // 风险过滤
        List<String> keysRisk = new ArrayList<>();
        keysRisk.add("RISK_SHIELD_FILTER_RULE_V1_JSON");
        keysRisk.add("RISK_SHIELD_FILTER_VIDEO_V1_STR");
        keysRisk.add("RISK_SHIELD_FILTER_EXPANSION_FACTOR_INT");
        keysRisk.add("RISK_SHIELD_FILTER_FLAG_BOOL");
        List<String> valuesRisk = redisTemplate.opsForValue().multiGet(keysRisk);
        String RISK_SHIELD_FILTER_RULE_V1_JSON = "";
        String RISK_SHIELD_FILTER_VIDEO_V1_STR = "";
        String RISK_SHIELD_FILTER_EXPANSION_FACTOR_INT = "";
        String RISK_SHIELD_FILTER_FLAG_BOOL = "";
        if (valuesRisk != null && valuesRisk.size() == 4){
            RISK_SHIELD_FILTER_RULE_V1_JSON = valuesRisk.get(0);
            RISK_SHIELD_FILTER_VIDEO_V1_STR = valuesRisk.get(1);
            RISK_SHIELD_FILTER_EXPANSION_FACTOR_INT = valuesRisk.get(2);
            RISK_SHIELD_FILTER_FLAG_BOOL = valuesRisk.get(3);
        }else{
            log.error("risk rule read redis is wrong");
        }
        Map<Integer, List<String>> appRegionFiltered = ParserUtils.parseJsonForRiskRule(RISK_SHIELD_FILTER_RULE_V1_JSON);
        List<Long> videosWithRisk = ParserUtils.parseJsonForRiskVideos(RISK_SHIELD_FILTER_VIDEO_V1_STR);
        int expansionFactor = 5;
        if (RISK_SHIELD_FILTER_EXPANSION_FACTOR_INT != null) {
            try {
                expansionFactor = Integer.parseInt(RISK_SHIELD_FILTER_EXPANSION_FACTOR_INT);
            } catch (NumberFormatException e) {
                expansionFactor = 5;
            }
        }
        // 容灾
        if (expansionFactor < 5) {
            expansionFactor = 5;
        } else if (expansionFactor > 25) {
            expansionFactor = 25;
        }
        Boolean riskFilterFlag = false;
        if (RISK_SHIELD_FILTER_FLAG_BOOL != null) {
            try {
                riskFilterFlag = Boolean.parseBoolean(RISK_SHIELD_FILTER_FLAG_BOOL.toLowerCase());
            } catch (Exception e) {
                riskFilterFlag = false;
            }
        }
        param.setAppRegionFiltered(appRegionFiltered);
        param.setVideosWithRisk(videosWithRisk);
        param.setExpansionFactor(expansionFactor);
        param.setRiskFilterFlag(riskFilterFlag);


        return param;
    }

    private List<Video> videoRecommend(RecommendParam param) {
        Stopwatch stopwatch = Stopwatch.createStarted();
        RecallResult recallResult = recallService.recall(convertToRecallParam(param));
        log.info("recallResult={}, videoRecommend recallResult cost={}", recallResult,
                stopwatch.elapsed(TimeUnit.MILLISECONDS));
        stopwatch.reset().start();

        RankResult rankResult = rankService.rank(convertToRankParam(param, recallResult));
        log.info("rankResult={}, videoRecommend rank cost={}", rankResult,
                stopwatch.stop().elapsed(TimeUnit.MILLISECONDS));


        if (rankResult == null || CollectionUtils.isEmpty(rankResult.getVideos())) {
            return Collections.emptyList();
        }

        // 只返回size条数据
        List<Video> videos = rankResult.getVideos();
        if (param.getSize() < rankResult.getVideos().size()) {
            videos = rankResult.getVideos().subList(0, param.getSize());
        }


        // mark video in flow pool
        // TODO 业务上有什么用处
        // 需要被标记的视频满足什么条件
        // 逻辑已经废弃
        // markFlowPoolVideo(param, videos);
        //log.info("markFlowPoolVideo videos {}", videos);

        return videos;
    }

    private RecallParam convertToRecallParam(RecommendParam param) {
        RecallParam recallParam = new RecallParam();
        recallParam.setAppType(param.getAppType());
        recallParam.setAbCode(param.getAbCode());
        recallParam.setVideoId(param.getVideoId());
        recallParam.setFlowPoolAbtestGroup(param.getFlowPoolAbtestGroup());

        String provinceCode = StringUtils.isNotBlank(param.getProvinceCode())
                ? param.getProvinceCode()
                : "";
        String cityCode = StringUtils.isNotBlank(param.getCityCode())
                ? param.getCityCode()
                : "";
        recallParam.setRegionCode(cityCodes.contains(cityCode) ? cityCode : provinceCode);

        recallParam.setRuleKey(param.getRuleKey());
        recallParam.setDataKey(param.getDataKey());
        recallParam.setMid(param.getMid());
        recallParam.setSize(param.getSize());
        recallParam.setUid(param.getUid());
        //风险过滤
        recallParam.setExpansionFactor(param.getExpansionFactor());
        recallParam.setRiskFilterFlag(param.getRiskFilterFlag());
        recallParam.setVideosWithRisk(param.getVideosWithRisk());
        recallParam.setAppRegionFiltered(param.getAppRegionFiltered());

        return recallParam;
    }


    private RankParam convertToRankParam(RecommendParam param, RecallResult recallResult) {
        RankParam rankParam = new RankParam();
        rankParam.setRecallResult(recallResult);
        rankParam.setAbCode(param.getAbCode());
        rankParam.setSize(param.getSize());
        rankParam.setFlowPoolP(param.getFlowPoolP());
        rankParam.setTopK(param.getTopK());
        rankParam.setRankKeyPrefix(param.getRankKeyPrefix());
        rankParam.setAppType(param.getAppType());

        // TODO RequestContext
        RequestContext context = new RequestContext();

        //rankParam.setRequestContext(context);
        return rankParam;
    }

    @Deprecated
    private void markFlowPoolVideo(RecommendParam param, List<Video> videos) {
        // TODO
        for (Video data : videos) {
            try {
                Long videoId = data.getVideoId();
                String quick_flow_pool_isin_flow_pool_key =
                        String.format("flow:pool:quick:video:ids:%s:3", param.getAppType());
                String quick_flow_pool_flow_pool_key =
                        String.format("flow:pool:quick:video:%s:3:%s", param.getAppType(), videoId);
                if (redisTemplate.opsForSet().isMember(quick_flow_pool_isin_flow_pool_key, String.valueOf(videoId))) {
                    data.setFlowPool(redisTemplate.opsForSet().randomMember(quick_flow_pool_flow_pool_key));
                    data.setInFlowPool(true);
                } else {
                    String isIn_flow_pool_key =
                            String.format("flow:pool:video:ids:%s", param.getAppType());
                    String flow_pool_key =
                            String.format("flow:pool:video:%s:%s", param.getAppType(), videoId);
                    if (redisTemplate.opsForSet().isMember(isIn_flow_pool_key, String.valueOf(videoId))) {
                        data.setFlowPool(redisTemplate.opsForSet().randomMember(flow_pool_key));
                        data.setInFlowPool(true);
                    }
                }
            } catch (Exception e) {
                log.error("markFlowPoolVideo error data={}", JSONUtils.toJson(data));
            }
        }
    }

    private void updateCache(RecommendRequest request, RecommendParam param, List<Video> videos) {
        if (StringUtils.isBlank(request.getMid())
                || CollectionUtils.isEmpty(videos)) {
            return;
        }

        preViewedService.updateCache(request.getAppType(), request.getMid(), videos);
        updateLastVideoCache(videos);

        updateFlowPoolCache(request, param, videos);

    }

    private void updateFlowPoolCache(RecommendRequest request, RecommendParam param,
                                     List<Video> videos) {
        if (request.getAppType() == 18
                || request.getAppType() == 19
                || CollectionUtils.isEmpty(videos)) {
            return;
        }

        List<Video> flowPoolVideos = videos.stream()
                .filter(v -> v.getPushFrom().equals(FlowPoolConstants.PUSH_FORM))
                .collect(Collectors.toList());


        if (CollectionUtils.isEmpty(flowPoolVideos)) {
            return;
        }

        switch (param.getFlowPoolAbtestGroup()) {
            case FlowPoolConstants.EXPERIMENTAL_FLOW_SET_LEVEL:
                flowPoolService.updateDistributeCountWithLevel(flowPoolVideos);
                break;
            case FlowPoolConstants.EXPERIMENTAL_FLOW_SET_LEVEL_SCORE:
                flowPoolService.updateDistributeCountWithLevelScore(flowPoolVideos);
                break;
            default:
                flowPoolService.updateDistributeCountWithScore(flowPoolVideos);
                break;
        }
    }

    private void updateLastVideoCache(List<Video> videos) {

        Consumer<String> consumer = (p) -> {
            for (int i = videos.size() - 1; i >= 0; i--) {
                if (videos.get(i).getPushFrom().equals(p)) {
                    redisTemplate.opsForValue().set(videos.get(i).getLastVideoKey(), String.valueOf(videos.get(i).getVideoId()),
                            24, TimeUnit.HOURS);
                    break;
                }
            }
        };

        consumer.accept(RegionHRecallStrategy.PUSH_FORM);
        consumer.accept(RegionHDupRecallStrategy.PUSH_FORM);
        consumer.accept(Region24HRecallStrategy.PUSH_FORM);
        consumer.accept(RegionRelative24HRecallStrategy.PUSH_FORM);
        consumer.accept(RegionRelative24HDupRecallStrategy.PUSH_FORM);
    }

}
