package com.tzld.piaoquan.recommend.server.service.recall.strategy;

import com.google.common.base.Stopwatch;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.filter.FilterResult;
import com.tzld.piaoquan.recommend.server.service.filter.FlowPoolWithLevelFilterService;
import com.tzld.piaoquan.recommend.server.service.recall.FilterParamFactory;
import com.tzld.piaoquan.recommend.server.service.recall.RecallParam;
import com.tzld.piaoquan.recommend.server.service.recall.RecallStrategy;
import com.tzld.piaoquan.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.RandomUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
@Slf4j
public abstract class AbstractFlowPoolWithLevelRecallStrategy implements RecallStrategy {
    @Autowired
    protected RedisTemplate<String, String> redisTemplate;

    @Autowired
    protected FlowPoolWithLevelFilterService filterService;

    @Override
    public List<Video> recall(RecallParam param) {
        Pair<String, String> flowPoolKeyAndLevel = flowPoolKeyAndLevel(param);
        String flowPoolKey = flowPoolKeyAndLevel.getLeft();
        String level = flowPoolKeyAndLevel.getRight();
        int getSize = param.getSize() * 5;
        List<Video> results = new ArrayList<>();
        log.info("flowPoolKeyAndLevel={}", JSONUtils.toJson(flowPoolKeyAndLevel));
        List<String> data = redisTemplate.opsForSet().randomMembers(flowPoolKey, getSize);
        log.info("randomMembers flowPoolKey={}, data={}", flowPoolKey, JSONUtils.toJson(data));
        if (CollectionUtils.isEmpty(data)) {
            return null;
        }
        Map<Long, String> videoFlowPoolMap = new HashMap<>();
        for (String value : data) {
            String[] values = value.split("-");
            videoFlowPoolMap.put(NumberUtils.toLong(values[0], 0), values[1]);
        }

        FilterResult filterResult = filterService.filter(FilterParamFactory.create(param, videoFlowPoolMap));

        if (filterResult != null && CollectionUtils.isNotEmpty(filterResult.getVideoIds())) {
            filterResult.getVideoIds().stream().forEach(vid -> {
                Video recallData = new Video();
                recallData.setVideoId(vid);
                recallData.setAbCode(param.getAbCode());
                recallData.setRovScore(RandomUtils.nextInt(0, 100));
                recallData.setPushFrom(pushFrom());
                recallData.setFlowPool(videoFlowPoolMap.get(vid));
                recallData.setFlowPoolAbtestGroup(param.getFlowPoolAbtestGroup());
                recallData.setLevel(level);
                results.add(recallData);
            });
        }

        return results.subList(0, results.size() < param.getSize() ? results.size() : param.getSize());
    }

    abstract Pair<String, String> flowPoolKeyAndLevel(RecallParam param);

}
