package com.tzld.piaoquan.recommend.server.service.score.feature;


import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.nio.charset.Charset;

public class FeatureHash {
    public static Charset CharSetUTF8 = Charset.forName("UTF-8");

    public static long getUInt32(byte a, byte b, byte c, byte d) {
        return (d << 24 | (c & 0xFF) << 16 | (b & 0xFF) << 8 | (a & 0xFF));
    }

    public static long hash64(byte[] data) {
        return MurmurHash64A(ByteBuffer.wrap(data), 0, data.length, 11L);
    }

    public static long MurmurHash64A(ByteBuffer buffer, int from, int len, long seed) {
        final long m = 0xc6a4a7935bd1e995L;
        final int r = 47;

        long h = (seed) ^ (len * m);
        int longLength = len / 8;

        for (int i = 0; i < longLength; ++i) {
            final int bytePos = from + i * 8;
            long k = buffer.getLong(bytePos);

            k *= m;
            k ^= k >> r;
            k *= m;
            h ^= k;
            h *= m;
        }

        final int remainingPos = len & ~7;
        switch (len % 8) {
            case 7: h ^= (long)(buffer.get(remainingPos + 6) & 0xFF) << 48;
            case 6: h ^= (long)(buffer.get(remainingPos + 5) & 0xFF) << 40;
            case 5: h ^= (long)(buffer.get(remainingPos + 4) & 0xFF) << 32;
            case 4: h ^= (long)(buffer.get(remainingPos + 3) & 0xFF) << 24;
            case 3: h ^= (long)(buffer.get(remainingPos + 2) & 0xFF) << 16;
            case 2: h ^= (long)(buffer.get(remainingPos + 1) & 0xFF) << 8;
            case 1:
                h ^= (long)(buffer.get(remainingPos) & 0xFF);
                h *= m;
        }

        h ^= h >>> r;
        h *= m;
        h ^= h >>> r;
        return h;
    }

    public static long MurmurHash32(byte data[], int len, long seed) {
        long m = 0x5bd1e995L;
        int r = 24;

        long h = seed ^ len;

        int offset = 0;
        while (len >= 4) {
            long k = getUInt32(data[offset], data[offset + 1], data[offset + 2], data[offset + 3]);

            k *= m;
            k &= 0xFFFFFFFFL;
            k ^= k >> r;
            k *= m;
            k &= 0xFFFFFFFFL;

            h *= m;
            h &= 0xFFFFFFFFL;
            h ^= k;

            offset += 4;
            len -= 4;
        }

        // Handle the last few bytes of the input array
        switch (len) {
            case 3: h ^= data[offset + 2] << 16;
            case 2: h ^= data[offset + 1] << 8;
            case 1: h ^= data[offset];
                h *= m;
                h &= 0xFFFFFFFFL;
        } ;

        // Do a few final mixes of the hash to ensure the last few
        // bytes are well-incorporated.

        h ^= h >> 13;
        h *= m;
        h &= 0xFFFFFFFFL;
        h ^= h >> 15;

        return h;
    }

    // 64-bit hash for 32-bit platforms
    public static long MurmurHash64(byte[] buffer, int start, int len, long seed) {
        final long m = 0x5bd1e995L;
        final int r = 24;
        final int original = len;

        long h1 = (seed) ^ len;
        long h2 = (seed >> 32);

        int offset = start;
        while (len >= 8) {
            long k1 = getUInt32(buffer[offset], buffer[offset + 1], buffer[offset + 2], buffer[offset + 3]);
            // long k1 = buffer.getInt(offset);

            k1 *= m; k1 &= 0xFFFFFFFFL; k1 ^= k1 >> r; k1 *= m; k1 &= 0xFFFFFFFFL;
            h1 *= m; h1 &= 0xFFFFFFFFL; h1 ^= k1;
            offset += 4;

            long k2 = getUInt32(buffer[offset], buffer[offset + 1], buffer[offset + 2], buffer[offset + 3]);
            // long k2 = buffer.getInt(offset);
            k2 *= m; k2 &= 0xFFFFFFFFL; k2 ^= k2 >> r; k2 *= m; k2 &= 0xFFFFFFFFL;
            h2 *= m; h2 &= 0xFFFFFFFFL; h2 ^= k2;

            offset += 4;
            len -= 8;
        }

        if (len >= 4) {
            long k1 = getUInt32(buffer[offset], buffer[offset + 1], buffer[offset + 2], buffer[offset + 3]);
            // long k1 = buffer.getInt(offset);
            k1 *= m; k1 &= 0xFFFFFFFFL; k1 ^= k1 >> r; k1 *= m; k1 &= 0xFFFFFFFFL;
            h1 *= m; h1 &= 0xFFFFFFFFL; h1 ^= k1;
            offset += 4;
            len -= 4;
        }

        switch (len) {
            case 3: h2 ^= (buffer[offset + 2] & 0xFF) << 16;
            case 2: h2 ^= (buffer[offset + 1] & 0xFF) << 8;
            case 1: h2 ^= (buffer[offset] & 0xFF);
                h2 *= m;
                h2 &= 0xFFFFFFFFL;
        } ;

        h1 ^= h2 >> 18;
        h1 *= m; h1 &= 0xFFFFFFFFL;
        h2 ^= h1 >> 22;
        h2 *= m; h2 &= 0xFFFFFFFFL;
        h1 ^= h2 >> 17;
        h1 *= m; h1 &= 0xFFFFFFFFL;
        h2 ^= h1 >> 19;
        h2 *= m; h2 &= 0xFFFFFFFFL;

        /*BigInteger ans = BigInteger.valueOf(h1).shiftLeft(32).or(BigInteger.valueOf(h2));
        return ans.longValue();*/
        //System.err.println("feature: " + new String(buffer, 0, original) + " length: " + original + " hash: " + (h1 << 32 | h2) + " daze");
        return h1 << 32 | h2;
    }

    // 64-bit hash for 32-bit platforms
    public static BigInteger MurmurHash64(byte data[], int len, long seed) {
        long m = 0x5bd1e995L;
        int r = 24;

        long h1 = (seed) ^ len;
        long h2 = (seed >> 32);

        int offset = 0;
        while (len >= 8) {
            long k1 = getUInt32(data[offset], data[offset + 1], data[offset + 2], data[offset + 3]);
            k1 *= m; k1 &= 0xFFFFFFFFL; k1 ^= k1 >> r; k1 *= m; k1 &= 0xFFFFFFFFL;
            h1 *= m; h1 &= 0xFFFFFFFFL; h1 ^= k1;

            long k2 = getUInt32(data[offset + 4], data[offset + 5], data[offset + 6], data[offset + 7]);
            k2 *= m; k2 &= 0xFFFFFFFFL; k2 ^= k2 >> r; k2 *= m; k2 &= 0xFFFFFFFFL;
            h2 *= m; h2 &= 0xFFFFFFFFL; h2 ^= k2;

            offset += 8;
            len -= 8;
        }

        if (len >= 4) {
            long k1 = getUInt32(data[offset], data[offset + 1], data[offset + 2], data[offset + 3]);
            k1 *= m; k1 &= 0xFFFFFFFFL; k1 ^= k1 >> r; k1 *= m; k1 &= 0xFFFFFFFFL;
            h1 *= m; h1 &= 0xFFFFFFFFL; h1 ^= k1;
            offset += 4;
            len -= 4;
        }

        switch (len) {
            case 3: h2 ^= (data[offset + 2] & 0xFF) << 16;
            case 2: h2 ^= (data[offset + 1] & 0xFF) << 8;
            case 1: h2 ^= (data[offset] & 0xFF);
                h2 *= m;
                h2 &= 0xFFFFFFFFL;
        } ;

        h1 ^= h2 >> 18;
        h1 *= m; h1 &= 0xFFFFFFFFL;
        h2 ^= h1 >> 22;
        h2 *= m; h2 &= 0xFFFFFFFFL;
        h1 ^= h2 >> 17;
        h1 *= m; h1 &= 0xFFFFFFFFL;
        h2 ^= h1 >> 19;
        h2 *= m; h2 &= 0xFFFFFFFFL;

        BigInteger ans = BigInteger.valueOf(h1).shiftLeft(32).or(BigInteger.valueOf(h2));
        return ans;
    }

    public static String hash(String input) {
        byte[] tt = input.getBytes(CharSetUTF8);
        return MurmurHash64(tt, tt.length, 11L).toString();
    }

    public static Long hashToLong(String input) {
        byte[] tt = input.getBytes(CharSetUTF8);
        return MurmurHash64(tt, tt.length, 11L).longValue();
    }

    /** the constant 2^64 */
    private static final BigInteger TWO_64 = BigInteger.ONE.shiftLeft(64);

    public static String asUnsignedLongString(long l) {
        BigInteger b = BigInteger.valueOf(l);
        if (b.signum() < 0) {
            b = b.add(TWO_64);
        }
        return b.toString();
    }
}
