package com.tzld.piaoquan.recommend.server.service.score.feature;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.ListMultimap;

import com.tzld.piaoquan.recommend.server.common.base.*;
import com.tzld.piaoquan.recommend.server.common.enums.VlogFeatureGroup;
import com.tzld.piaoquan.recommend.server.gen.recommend.LRSamples;
import com.tzld.piaoquan.recommend.server.gen.recommend.BaseFeature;
import com.tzld.piaoquan.recommend.server.gen.recommend.RecommendRequest;
import com.tzld.piaoquan.recommend.server.gen.recommend.FeatureGroup;

import java.util.List;


public abstract class LRBytesFeatureExtractorBase {
    private static final double DEFAULT_USER_CTR_GROUP = 10.0;
    private static final double DEFAULT_ARTICLE_CTR_GROUP = 100.0;


    private BytesUtils utils;
    //Feature Group & Features
    ListMultimap<FeatureGroup, BaseFeature> features = ArrayListMultimap.create();
    int groupCount;


    LRBytesFeatureExtractorBase() {
        groupCount = VlogFeatureGroup.values().length;
        BytesGroup[] groups = new BytesGroup[groupCount];
        for (VlogFeatureGroup g: VlogFeatureGroup.values()) {
            groups[g.ordinal()] = new BytesGroup(g.ordinal(),
                    g.getGroupName(), g.getGroupNameBytes());
        }
        utils = new BytesUtils(groups);
    }

    private FeatureGroup makeGroup(VlogFeatureGroup group){
        FeatureGroup.Builder g = FeatureGroup.newBuilder();
        g.setType("1");
        g.setName(group.getGroupName());
        g.setId(group.ordinal());
        return g.build();
    };


    void makeFea(VlogFeatureGroup group, byte[] value) {
        FeatureGroup featureGroup = makeGroup(group);
        BaseFeature feature = utils.makeFea(group.ordinal(), value);
        features.put(featureGroup, feature);
    }

    void makeFea(VlogFeatureGroup group, byte[][] list) {
        FeatureGroup g = makeGroup(group);
        List<BaseFeature> featureList = utils.makeFea(group.ordinal(), list);
        features.putAll(g, featureList);
    }

    public ListMultimap<FeatureGroup, BaseFeature> getFeatures() {
        return features;
    }

    public abstract LRSamples single(UserBytesFeature userBytesFeature,
                                     VideoBytesFeature videoBytesFeature,
                                     RequestContextBytesFeature requestContextBytesFeature);


}
